#!/usr/bin/ruby
require "hdatastructures/hrecord"

class Array

  alias_method :insertFirst, :unshift
  alias_method :insertLast,  :<<
  alias_method :deleteFirst, :shift
  alias_method :deleteLast,  :pop
  alias_method :deleteAt,    :delete_at
  alias_method :indexOf,     :index

  def addIfNotPresent(value)

    self << value unless self.include?(value)
    return value

  end

  def deleteAt(index)
    self.delete_at(index) if index 
  end

  def swap(index1, index2)
    # Parallel Assignment: eg a,b = 1,2
    self[index1], self[index2] = self[index2], self[index1]
  end

  def < (array)
    for i in 0...self.count
      return true unless self[i]
      return false unless array[i]
      return true if self[i] < array[i]
      return false if self[i] > array[i]
    end
    return false
  end
  
  def hpartition
    return self.select { |value| value if yield(value) }, self.select { |value| value unless yield(value) }
  end
  
  def herbertsort(asc = true, &block)

    #block = lambda { |data| data } unless block
    block = Proc.new { |data| data } unless block

    return [] if empty?
    pivot = delete_at(rand(size))
    left, right = hpartition { |value| block.call(value) < block.call(pivot) } if asc
    left, right = hpartition { |value| !(block.call(value) < block.call(pivot)) } unless asc 
    return *left.herbertsort(asc, &block), pivot, *right.herbertsort(asc, &block)

  end

  # [4,2,1,5,3].herbertsort => [1, 2, 3, 4, 5]
  # [[4,0],[2,0],[1,0],[5,0],[3,0]].herbertsort => [[1, 0], [2, 0], [3, 0], [4, 0], [5, 0]]
  # [[4,1],[2,0],[1,0],[4,0],[3,0]].herbertsort => [[1, 0], [2, 0], [3, 0], [4, 0], [4, 1]]
  # Sorting by qty
  # [{qty:1, price:3}, {qty:2, price:2}, {qty:3, price:1}].herbertsort { |data| data[:qty] } =>
  #                            [{:qty=>1, :price=>3}, {:qty=>2, :price=>2}, {:qty=>3, :price=>1}]
  # Sorting by qty and by price                           
  # [{qty:1, price:3}, {qty:1, price:2}, {qty:1, price:1}].herbertsort { |data| [data[:qty], data[:price]] }
  def herbertsort!(asc = true, &block)
    self.herbertsort(asc, &block).each_with_index { |data, i| self[i] = data }
  end

  def show(key = :key)
    self.each do |data|
      if data.class == HRecord
        data.show(key) 
      else
        puts data.to_s
      end
    end
  end

  def self.test()
    list = []
    #(20).downto(1) { |i|
    for i in 0..9
      list.insertLast(HRecord.new(i))
    end
    list.deleteFirst()
    list.deleteLast()
    list.deleteAt(2)
    list.insert(0, HRecord.new(0))
    list.insert(3, "ciao")
    list.insertLast(HRecord.new(9))
    list[15] = 20
    list[16] = :end
    list.swap(15, 16)
    list.show()
    p list.indexOf(HRecord.new(9))
    p [1,6,5,4,3,23,2].herbertsort
  end

end

#Array.test()
