class Hash

  def join(separator)

    arr = []
    self.each do |key, value|
      arr << "\"#{key}\" = #{value}"
    end
    return arr.join(separator)

  end

  def intersect(hash)

    self.select {|k,v| hash.include?(k)}

  end
  
  def hprint

    hashView = self.dup
    #'*ptr'
    hashView.each { |k, v| hashView[k] = v.object_id if v.class != String and v.class != Fixnum and v.class != NilClass and v.class != Float }
    puts hashView

  end

  def to_js_format
    result = []
    self.each do |k, v|
      tmp = (v.class <= String) ? "#{k}:#{v.to_js_format}" : "#{k}:#{v}"
      result << tmp
    end
    return "{#{result.join(", ")}}"
  end


end

class Array

  def hjoin(separator, quoteChar = '"')

    arr = []
    self.each { |value| arr << "#{quoteChar}#{value}#{quoteChar}" }
    return arr.join(separator)
  end
  
  def to_js_format # used in HWidget::buildSignature
    result = []
    self.each do |value|
      tmp = (value.class <= String) ? "#{value.to_js_format}" : value
      result << tmp
    end
    return "[#{result.join(", ")}]"
  end

end

class String

  def hcapitalize

    self.split('_').map(&:capitalize) * ''

  end

  def to_js_format
    return "'#{self.gsub("'", "\\\\'")}'"
  end

  def to_b
    return self == "true"
  end

end

class HDB 

  attr_reader :resultTable, :connection, :sth, :table

  def initialize(host, port, dbname, user, password, timezone, connectionName = "default", connector = "Pg")

    @host = host
    @port = port
    @dbname = dbname
    @user = user
    @password = password
    @timezone = timezone
    @connectionName = connectionName
    
    @connection = nil

    @select = nil
    @from = nil
    @where = nil
    @orderBy = nil
    @direction = nil
    @limit = nil
    @offset = nil

    @resultTable = nil
    @table = nil

  end

  def method_missing(methodeName, *args)
    return eval("HODB.new($1, @connectionName).find_by_#{$2}(*args)") if methodeName.to_s =~ /^find_on_(.+)_by_(.+)$/
  end

  # posso passare anche un array di campi
  def select(select)
    select = select.hjoin(', ') if(select.class == Array)
    @select = select
    return self
  end
  def from(from)
    from = from.hjoin(', ') if(from.class == Array)
    @from = from
    return self
  end
  def normalizeWhereFormat(where, operator = 'AND')
    where = self.quote(where) if where.class == Hash
    where = where.join(" #{operator} ") if(where.class == Array || where.class == Hash)
    return where
  end
  def where(where, operator = 'AND')
    @where = self.normalizeWhereFormat(where, operator)
    return self
  end
  def orderBy(orderBy)
    orderBy = orderBy.hjoin(', ') if(orderBy.class == Array)
    @orderBy = orderBy
    return self
  end
  def direction(direction)
    @direction = direction
    return self
  end
  def limit(limit)
    @limit = limit
    return self
  end
  def offset(offset)
    @offset = offset
    return self
  end
  
  def queryStr()
 
    where = "WHERE #{@where}" if(@where)
    orderBy  = "ORDER BY #{@orderBy}" if(@orderBy)
    limit = "LIMIT #{@limit}" if(@limit)
    offset = "OFFSET #{@offset}" if(@offset)
    return "SELECT #{@select} FROM #{@from} #{where} #{orderBy} #{@direction} #{limit} #{offset}"

  end


  def openConnection()

    hl << "connectionName: #{@connectionName}".red
    hl << "host: #{@host}".red
    hl << "port: #{@port}".red
    hl << "db: #{@dbname}".red
    hl << "user: #{@user}".red
    hl << "password: #{@password}".red
    hl << "timezone: #{@timezone}".red
    @connection = self.connect() unless @connection

  end


  def closeConnection()
    
    self.disconnect() if @connection
    @connection = nil
  
  end
  
  
  def execute(queryStr = self.queryStr, pageSize: "all", page: 0) 
    
    limit = "LIMIT #{pageSize}"
    offset = "OFFSET #{page} * #{pageSize}"
    if(pageSize == "all")
      limit = offset = ""
    end

    queryStr += " #{limit} #{offset}"
    
    t1 = Time.now
    hl << queryStr.hight_yellow
    result = self._execute(queryStr)
    hl << "... msecs. #{((Time.now - t1) * 1000).round(2)}".hight_cyan
    return result

  end
  
  def quoteValue(value)
      
      return 'NULL' if value == nil
      value = value.to_s.gsub("'", "''")
      return (value[0] != '#') ? "'#{value}'" : value[1, value.size - 1]

  end

  def primitiveType?(value)

    return (value.class == String or
      value.class == Fixnum or
      value.class == TrueClass or
      value.class == FalseClass)

  end

  # quota tutti i valori tranne quelli che iniziano con #
  # quindi se nella mia query devo richiamare una funzione basta farla precedere dal cancelletto
  def quote(args)
    
    return 'NULL' if args == nil
    return self.quoteValue(args) if self.primitiveType?(args) 
    
    result = Hash.new()

    args.each do |key, value|
      result[key] = self.quoteValue(args[key])
    end
    return result

  end

  alias_method :q, :quote

  def toClassName(tableName)
    return tableName.hcapitalize()
  end

  def insertValues(args) 

    return "(id) values(default)" unless(args)
    args = self.quote(args)
    return "(#{args.keys.hjoin(', ')}) values(#{args.values.join(', ')})"

  end

  def insert(tableName, values)

    return self.execute("INSERT INTO \"#{tableName}\" #{self.insertValues(values)} RETURNING id").data(0,"id") 

  end

  def updateValues(args) 

    return "" unless(args)
    args = self.quote(args)
    result = []
    args.each { |key, value| result << "\"#{key}\" = #{value}" }  
    return result.join(', ')

  end
  
  def update(tableName, values, where = "TRUE", operator = "AND")

    where = where.join(" #{operator} ") if(where.class == Array || where.class == Hash)
    self.execute("UPDATE \"#{tableName}\" SET #{self.updateValues(values)} WHERE #{where}") 
    return self.rowsAffected()

  end
  
  def delete(tableName, where = "TRUE", operator = "AND")
    
    where = self.quote(where) if where.class == Hash
    where = where.join(" #{operator} ") if(where.class == Array || where.class == Hash)
    self.execute("DELETE FROM \"#{tableName}\" WHERE #{where}") 
    return self.rowsAffected() # rows affected

  end
  
  def rowCount()

    return self.rowsAffected()

  end

  def columnCount()

    return self.fieldNameList.length

  end

  def data(row, idOrFieldName)

    return (idOrFieldName.class == Fixnum) ? @table[row][@table[row].keys[idOrFieldName]] : @table[row][idOrFieldName.to_s]

  end

  def show()
  
    @table.each do |row|
      p row
    end

  end

  def self.newHDB(connectionName = "default")

    params = hc.value(connectionName)
    
    host   = params["host"]
    port   = params["port"]
    dbname = params["dbname"]
    user   = params["user"]
    password = params["password"]
    timezone = params["timezone"]
    connector = params["connector"]

    connector = connector.split('-') 

    if (connector[0] == "hdbi")
      hdb = HDBI.new(host, port, dbname, user, password, timezone, connectionName, connector[1])
    elsif (connector[0] == "hpgsql")
      hdb = HPgSql.new(host, port, dbname, user, password, timezone, connectionName)
    elsif (connector[0] == "hmysql")
      hdb = HMySql.new(host, port, dbname, user, password, timezone, connectionName)
    elsif (connector[0] == "hmysql2")
      hdb = HMySql2.new(host, port, dbname, user, password, timezone, connectionName)
    end

    hdb.openConnection()
    HSharedData.instance().setValue(hdb, "hdb-#{connectionName}")
    return hdb

  end

  def self.HDB(connectionName = "default")

    hdb = HSharedData.instance().value("hdb-#{connectionName}")
    return (hdb) ? hdb : self.newHDB(connectionName) 

  end

  def self.destroy(connectionName = "default")
    
    self.HDB(connectionName).closeConnection()
    HSharedData.instance().setValue(nil, "hdb-#{connectionName}")

  end

end

def hdb(connectionName = "default")

  return HDB.HDB(connectionName)

end

