#!/usr/bin/ruby
require 'dbi'
require 'hdb/hdb'

class HDBI < HDB

  def initialize(host, port, dbname, user, password, timezone, connectionName, connector = "Pg")

    super(host, port, dbname, user, password, timezone, connectionName, connector)
    @connector = connector
    @sth = nil

  end
 
  def connect()

    @connection = DBI.connect("dbi:#{@connector}:#{@dbname}:#{@host}", @user, @password)
    hl << "Server version: #{self.execute("SHOW server_version").firstData.to_s}"
    self.execute("SET TIME ZONE '#{@timezone}'") if @timezone
    return @connection

  end

  def disconnect()

    @connection.disconnect()
    @connection = nil 

  end
  
  def _execute(queryStr = self.queryStr) 

    @sth = @connection.execute(queryStr)
    result = @sth.fetch_all
    @resultTable = HFieldTable.new()
    self.fieldNameList().each { |fieldName| @resultTable.addFieldName(fieldName) }
    @resultTable.makeCaption()
    result.each_with_index do |row, i|  
      row.each_with_name do |fieldValue, fieldName|
        @resultTable.setDataByFieldName(i, fieldName, fieldValue)
      end
    end
    @sth.finish
    @sth = nil
    return self

  end
  
  def fieldNameList()

    fieldList = HList.new()
    
    @sth.column_names.each { |fieldName| fieldList << fieldName }
    
    return fieldList

  end
 
  def rowsAffected
    return @sth.rows # rows affected
  end
end

