#!/usr/bin/ruby

=begin
FIELDS_TO_PGTYPES = {
    fields.boolean: 'bool',
    fields.integer: 'int4',
    fields.text: 'text',
    fields.html: 'text',
    fields.date: 'date',
    fields.datetime: 'timestamp',
    fields.binary: 'bytea',
    fields.many2one: 'int4',
    fields.serialized: 'text',
}
=end

class HField < Hash

  def initialize(args)
    args.each { |key, value| self[key] = value }
  end

  def defaultValue()
    
    case self[:type]
    when "char"
      return 'true'
    when "text"
      return ''
    when "integer"
      return 0
    when "float"
      return 0
    when "date"
      return '' 
    when "dateTime"
      return ''
    when "binary"
      return ''
    when "selection"
      return ''
    when "boolean"
      return 'true'
    else
      return nil
    end

  end

  def self.autoComplete(modelName, fieldName, args = {})
    
    return args if args[:type] != "automatic"
    
    if fieldName =~ /^(\w+)_id$/
      return self.manyToOne($1, args)
    elsif fieldName =~ /^(\w+)_table$/
      return self.oneToMany($1, "#{modelName}_id", args)
    elsif fieldName =~ /^(\w+)_join$/
      return self.manyToMany($1, "#{modelName}_#{fieldName}", "#{modelName}_id", "#{$1}_id", args)
    elsif fieldName =~ /^(\w+)_virtual$/
      return self.virtual($1, nil, args)
    end

    return HField.new(args)

  end
  
  def self.automatic(args = {})
    args[:type] = "automatic"
    return HField.new(args)
  end
  
  def self.magic(args = {})
    return self.automatic(args)
  end

  def self.char(args = {})
    args[:type] = "char"
    args[:default] = '' unless args[:default]
    return HField.new(args)
  end

  def self.text(args = {})
    args[:type] = "text"
    args[:default] = '' unless args[:default]
    return HField.new(args)
  end
  
  def self.integer(args = {})
    args[:type] = "integer"
    args[:default] = 0 unless args[:default]
    return HField.new(args)
  end
  
  def self.float(args = {})
    args[:type] = "float"
    args[:default] = 0 unless args[:default]
    return HField.new(args)
  end
  
  def self.date(args = {})
    args[:type] = "date"
    args[:default] = '' unless args[:default]
    return HField.new(args)
  end
  
  def self.dateTime(args = {})
    args[:type] = "datetime"
    return HField.new(args)
  end
  
  def self.binary(args = {})
    args[:type] = "binary"
    args[:default] = '' unless args[:default]
    return HField.new(args)
  end
  
  def self.selection(args = {})
    args[:type] = "selection"
    return HField.new(args)
  end
  
  def self.boolean(args = {})
    args[:type] = "boolean"
    args[:default] = false unless args[:default]
    return HField.new(args)
  end
  
  def self.oneToOne(modelName, args = {})
    args[:modelNameReference] = modelName
    #args[:constraint] = 'NOT NULL'
    args[:onDelete] = 'CASCADE' # CASCADE or RESTRICT
    args[:onUpdate] = 'CASCADE'
    args[:type] = "oneToOne"
    return HField.new(args)
  end
  
  def self.oneToMany(modelName, referenceFieldName, args = {})
    args[:modelNameReference] = modelName
    args[:referenceFieldName] = referenceFieldName
    args[:type] = "oneToMany"
    return HField.new(args)
  end
  
  def self.manyToOne(modelName, args = {})
    args[:modelNameReference] = modelName
    args[:onDelete] = 'CASCADE'
    args[:onUpdate] = 'CASCADE'
    #args[:constraint] = 'NOT NULL'
    args[:type] = "manyToOne"
    return HField.new(args)
  end
  
  def self.manyToMany(modelName, joinTable, referenceFieldName, referenceFieldName2, args = {})
    args[:modelNameReference] = modelName
    args[:joinTable] = joinTable
    args[:referenceFieldName] = referenceFieldName
    args[:referenceFieldName2] = referenceFieldName2
    #args[:constraint] = 'NOT NULL'
    args[:type] = "manyToMany"
    return HField.new(args)
  end

  def self.virtual(functionName = nil, object = nil, args = {}, &block)
    args[:object] = object
    args[:functionName] = functionName
    args[:type] = "virtual"
    args[:block] = block
    return HField.new(args)
  end
  
  def self.primaryKey(args = {})
    args[:type] = 'integer'
    args[:system] = true
    args[:constraint] = 'SERIAL PRIMARY KEY'
    return HField.new(args)
  end

end

