#!/usr/bin/ruby

require "singleton"
require "nokogiri"

class Nokogiri::XML::Element

  # <panel name="article">
  #   <panel name="product" caption="Product" position="inside">
  #     <field name="hello" caption="Name"/>
  #   </panel>
  # </panel>

  # If node is <field name="name" caption="Name"/> then:
  # - node.value("name") # => hello
  # - node.value("caption") # => Name
  # e se voglio in name di panel ?
  # - node.value("panel.name") # => product
  # - node.value("panel.panel.name") # => article

  def value(name)
    tokens = name.split('.')
    return nil if tokens.empty?
    return self.parent.value(tokens[1..-1].join('.')) if tokens.length > 2
    # Se per es. name e' "panel.label" e arriviamo all'elemento panel si deve restituire il valore di label 
    # e se tale elemento non esiste si restituisce nil ma non si deve continare a ricercare tra i genitori
    return self[tokens[1]] if (tokens.length == 2 and tokens[0] == self.name)
    return self[name] if self[name] 
    return (self.parent and self.parent.class == Nokogiri::XML::Element) ? self.parent.value(name) : nil
  end 

end

class HViewLoader

  include Singleton 

  def initialize()
    @hviews = {} # hypersonic views
  end
  
  def viewByFileName(moduleName, filename)
    
    viewName = filename 
    @hviews[moduleName] = {} unless @hviews[moduleName]
    return @hviews[moduleName][viewName] if @hviews[moduleName][viewName]
    @hviews[moduleName][viewName] = Nokogiri::Slop(File.read(filename))

  end  

  def view(moduleName, viewName)

    return self.viewByFileName(moduleName, "#{Dir.pwd}/app/modules/#{moduleName}/#{viewName}/#{viewName}_view.xml")

  end  

  def value(fieldName, viewName)

    return self.view(viewName)[fieldName] 

  end
   
  def viewByType(moduleName, viewName, type)
    return self.view(moduleName, viewName).xpath("//view[@type='#{type}']//*")
  end

  def viewParsing(moduleName, viewName, type)

    nodes = []
    view = self.view(moduleName, viewName).xpath("//view[@type='#{type}']//*")
    #view = view.xpath(".//*")
    view.each do |node| 
      p node.name
      # node[:type]
      case node.name 
      when "table"
        puts "table"
      when "form"
        puts "form"
      when "panel"
        puts "panel"
      when "field"
        nodes << node          
      else
      end
    end

    return nodes

  end
  
end

def hv()

  return HViewLoader.instance()

end

